import copy
import traceback

from .smwasm import rs_load_wasm, rs_call, rs_register_native
from smwasm.core.smcore import USAGE, g_funcs, g_paths, g_usages, call_sm
from smwasm import smu


_g_logger = None


def register(itdef, path, func):
    name = itdef[USAGE]
    g_usages[name] = itdef
    g_funcs[name] = func
    g_paths[name] = path
    rs_register_native(smu.dict_to_format_json(itdef, 2))

def load_wasm(wasm_path, page_num):
    rs_load_wasm(wasm_path, page_num)


def call(dt):
    dtRet = call_sm(dt)
    if dtRet is None:
        try:
            txt = rs_call(smu.dict_to_format_json(dt, 2))
            dtRet = smu.json_to_dict(txt)
        except Exception as e:
            txt = "{0}\n{1}".format(str(e), traceback.format_exc())
            dtRet = {"$panic": txt}
    return dtRet


def call_native(intxt):
    dt = smu.json_to_dict(intxt)
    dtRet = call_sm(dt)
    if dtRet is None:
        return '{}'
    outtxt = smu.dict_to_format_json(dtRet, 2)
    return outtxt


def info():
    ret = {"function": g_paths}
    return ret


def log(text):
    if _g_logger:
        _g_logger.info(text)
    else:
        print(text)


def set_logger(logger):
    global _g_logger
    _g_logger = logger



import time
import json
import codecs
import os

try:
    import yaml
except:
    pass

from smwasm.base._py_base import pybase


def current():
    lt = time.gmtime(time.time() - time.timezone)
    return time.strftime("%Y-%m-%d %H:%M:%S", lt)


def dict_to_format_json(dictData, indent):
    txt = ""
    try:
        txt = json.dumps(dictData, ensure_ascii=False, indent=indent)
    except:
        pass
    return txt


def json_to_dict(text):
    if not text:
        return None

    dt = json.loads(text)
    return dt


def read_file(fileName):
    fileData = ""
    if os.path.exists(fileName):
        try:
            f = codecs.open(fileName, "r", "utf8")
            fileData = f.read()
            f.close()
        except:
            pass
    return fileData


def read_json(fileName):
    fileData = read_file(fileName)
    if fileData:
        try:
            dictData = pybase.load_json(fileData)
            return dictData
        except:
            try:
                dictData = yaml.load(fileData, Loader=yaml.FullLoader)
                return dictData
            except:
                try:
                    dictData = yaml.load(fileData)
                    return dictData
                except:
                    return None
    return None


def write_json(fileName, jsonData, codePage="utf8"):
    jsonString = json.dumps(jsonData, ensure_ascii=False, indent=2).encode(codePage)
    f = open(fileName, "wb")
    f.write(jsonString)
    f.close()



# --- python 2 special ---

import json
from collections import OrderedDict


class PyBase:

    @staticmethod
    def is_method(attr):
        type_name = type(attr).__name__
        return type_name == "instancemethod"

    @staticmethod
    def load_json(text, object_pairs_hook=OrderedDict):
        return json.loads(text, object_pairs_hook=object_pairs_hook)



import traceback

USAGE = "$usage"

g_funcs = {}
g_paths = {}
g_usages = {}

def call_sm(dt):
    usage = dt.get(USAGE)
    if usage in g_funcs:
        func = g_funcs.get(usage)
        try:
            dtRet = func(dt)
            return dtRet
        except Exception as e:
            txt = "{0}\n{1}".format(str(e), traceback.format_exc())
            return {"$panic": txt}
    return None



# --- python 3 special ---

import json
from collections import OrderedDict


class PyBase:

    @staticmethod
    def is_method(attr):
        type_name = type(attr).__name__
        return type_name == "method"

    @staticmethod
    def load_json(text, object_pairs_hook=OrderedDict):
        type_name = type(text).__name__
        if type_name == "bytes":
            text = text.decode("utf8")
        return json.loads(text, object_pairs_hook=object_pairs_hook)
